#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>

#include "test.h"

CU_pSuite suite_trie() {
    CU_pSuite suite = CU_add_suite("ptrie", 0, 0);

    CU_ADD_TEST(suite, test_trie_new);
    CU_ADD_TEST(suite, test_trie_various);
//    CU_ADD_TEST(suite, test_performance);

    return suite;
}

void test_trie_new() {
    ptrie *trie = 0;

    CU_ASSERT_EQUAL(PR_ERROR, ptrie_new(0, 0));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_new(0, &trie));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_new(free, 0));

    CU_ASSERT_EQUAL(PR_OK, ptrie_new(free, &trie));
    CU_ASSERT_NOT_EQUAL(0, trie);

    CU_ASSERT_EQUAL(PR_ERROR, ptrie_free(0));
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));
    CU_ASSERT_EQUAL(0, trie);
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_free(&trie));
}

void nothing(void *value) {
}

void test_trie_various() {
    ptrie *trie = 0;
    char *value = "123";
    char *value2[5];
    void *nvalue;
    unsigned int kl=0;
    ptrie_iterator *it=0;

    // 1.
    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get_length(trie, &kl));
    CU_ASSERT_EQUAL(0, kl);

    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "a", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get_length(trie, &kl));
    CU_ASSERT_EQUAL(1, kl);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterate(trie, &it));
    CU_ASSERT_NOT_EQUAL(0, it);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("a", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_free(&it));

    CU_ASSERT_EQUAL(PR_OK, ptrie_delete(trie, 1, "a"));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));

// 2.
    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 10, "abcdefghij", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 10, "abcdefghij", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_delete(trie, 10, "abcdefghij"));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_get(trie, 10, "abcdefghij", &nvalue));
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));

    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "a", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "b", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "c", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get_length(trie, &kl));
    CU_ASSERT_EQUAL(3, kl);

    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "b", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "c", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterate(trie, &it));
    CU_ASSERT_NOT_EQUAL(0, it);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("a", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("b", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("c", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_free(&it));

    CU_ASSERT_EQUAL(PR_OK, ptrie_delete(trie, 1, "a"));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "b", &nvalue));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "c", &nvalue));
    CU_ASSERT_EQUAL(PR_OK, ptrie_delete(trie, 1, "c"));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "b", &nvalue));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_get(trie, 1, "c", &nvalue));
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));

    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "a", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "a", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 2, "aa", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 3, "aaa", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 2, "aa", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 3, "aaa", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));

    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 2, "aa", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 6, "aabbbb", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 14, "aabbbbcccccccc", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 2, "aa", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 6, "aabbbb", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 14, "aabbbbcccccccc", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));

    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "a", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 2, "bb", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 3, "ccc", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 2, "bb", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 3, "ccc", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));

    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "c", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "b", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "a", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "b", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "c", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);

    // nth

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterate(trie, &it));
    CU_ASSERT_NOT_EQUAL(0, it);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("a", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("b", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("c", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_free(&it));

    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));

//nth

    CU_ASSERT_EQUAL(PR_OK, ptrie_new(nothing, &trie));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 1, "a", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 2, "ab", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 5, "abbbb", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 5, "aaaaa", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 5, "abbaa", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_set(trie, 3, "aaa", value));
    CU_ASSERT_EQUAL(PR_OK, ptrie_get_length(trie, &kl));
    CU_ASSERT_EQUAL(6, kl);

    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 1, "a", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 2, "ab", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 3, "aaa", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 5, "abbbb", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 5, "aaaaa", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);
    CU_ASSERT_EQUAL(PR_OK, ptrie_get(trie, 5, "abbaa", &nvalue));
    CU_ASSERT_EQUAL(nvalue, value);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterate(trie, &it));
    CU_ASSERT_NOT_EQUAL(0, it);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(1, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("a", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(3, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("aaa", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(5, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("aaaaa", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(2, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("ab", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(5, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("abbaa", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);

    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_key_length(it, &kl));
    CU_ASSERT_EQUAL(5, kl);
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_copy_key(it, value2));
    CU_ASSERT_EQUAL(0, strncmp("abbbb", value2, 1));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_get_value(it, &nvalue));
    CU_ASSERT_EQUAL(value, nvalue);

    CU_ASSERT_EQUAL(PR_ERROR, ptrie_iterator_next(it));
    CU_ASSERT_EQUAL(PR_OK, ptrie_iterator_free(&it));

    CU_ASSERT_EQUAL(PR_OK, ptrie_delete(trie, 3, "aaa"));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_delete(trie, 3, "aaa"));
    CU_ASSERT_EQUAL(PR_OK, ptrie_delete(trie, 1, "a"));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_delete(trie, 1, "a"));
    CU_ASSERT_EQUAL(PR_ERROR, ptrie_delete(trie, 3, "abb"));
    CU_ASSERT_EQUAL(PR_OK, ptrie_free(&trie));
}

#define DICTIONARY "ab"
#define DICTIONARY_LEN strlen(DICTIONARY)
#define LENGTH 2

typedef struct item item;
typedef struct combination combination;

void generator_init(unsigned long *,unsigned long);
void generator_next(unsigned long *,short,unsigned long max,unsigned long);
void generate_items(item ***,unsigned long *);

void free_items(item ***,unsigned long);
int item_cmp(item **,item **);
void *item_fake_free(void *);

void work_combinations(item **,unsigned long);
void work_combination(combination *,unsigned long,unsigned long);
void unduplicate_combination(combination *,combination **);
void sort_combination(combination *,combination **);

void error_1_cannot_set(combination *,item *);
void error_2_cannot_get(combination *,item *);
void error_3_1_cannot_delete(combination *,item *);
void error_3_2_did_get(combination *,item *);
void error_3_3_cannot_get(combination *,item *,item *);
void error_3_4_cannot_set(combination *,item *);

void concat(char **pthis, char *format, ...);
void combination_concat(char **pthis,combination *);
void item_concat(char **pthis,item *);

struct item {
    unsigned long kl;
    char *k;
};

struct combination {
    unsigned long len;
    item **items;
};

void generator_init(unsigned long *this, unsigned long len) {
    unsigned long i;
    for(i=0; i<len; i++) {
        this[i]=0;
    }

    printf("after init: ");
    for(i=0; i<len; i++) {
        printf("%d ",this[i]);
    }
    printf("\n");
}

void generator_next(unsigned long *this, short can_repeat,unsigned long max,unsigned long len) {
    unsigned long i,j;
    short r;

    if(0 == this) {
        return;
    }

    for(i = len - 1; i >= 0; i--) {
        r=0;
        this[i]++;

        if(can_repeat==0) {
            for(j=0; j<len; j++) {
                if(i==j) {
                    continue;
                }
                if(this[i]==this[j]) {
                    r=1;
                    break;
                }
            }
        }

        if(r==1) {
            continue;
        }

        if(this[i] != max) {
            break;
        }

        this[i]=0;
    }

    printf("after %d: ", max);
    for(i=0; i<len; i++) {
        printf("%d ",this[i]);
    }
    printf("\n");

}

void generate_items(item ***pthis,unsigned long *plen) {
    unsigned long it, j, pos, item_pos;
    unsigned long items_in_iteration;
    item **items;
    unsigned long len;
    unsigned long generator[LENGTH];

    len = 0;
    items_in_iteration = DICTIONARY_LEN;
    for(it=0; it<LENGTH; it++) {
        len += items_in_iteration;
        items_in_iteration *= DICTIONARY_LEN;
    }

    *plen=len;
    items = (item **)malloc(sizeof(struct item *) * len);
    items_in_iteration = DICTIONARY_LEN;

    printf("Will generate %d items\n", len);

    item_pos = 0;
    for(it=1; it<=LENGTH; it++) {
        generator_init(&generator, it);

        for(j=0; j<items_in_iteration; j++) {
            items[item_pos] = (item *)malloc(sizeof(struct item));
            items[item_pos]->kl = it;
            items[item_pos]->k=(char *)malloc(sizeof(char)*it);
            for(pos=0; pos<it; pos++) {
                items[item_pos]->k[pos] = DICTIONARY[generator[pos]];
            }

            item_pos++;
            generator_next(&generator, 1,DICTIONARY_LEN, it);
        }

        items_in_iteration *= DICTIONARY_LEN;
    }

    *pthis=items;
}

void free_items(item ***pthis,unsigned long len) {
    unsigned long item_pos;
    item **items=*pthis;

    for(item_pos=0; item_pos<len; item_pos++) {
        free(items[item_pos]->k);
        free(items[item_pos]);
    }

    free(items);
}

unsigned long p_n_k(unsigned long n,unsigned long k) {
    unsigned long i, ret=1;

    for(i=n-k+1; i<=n; i++) {
        ret*=i;
    }
    return ret;
}

void work_combinations(item **items, unsigned long items_len) {
    unsigned long it, j, pos, combination_pos;
    unsigned long combinations_in_iteration;
    unsigned long len;
    unsigned long *generator;
    combination *combo;

    generator = (unsigned long *)malloc(sizeof(unsigned long)*items_len);

    len = 0;
    for(it=1; it<=items_len; it++) {
        printf("%ld %ld\n", it, p_n_k(items_len, it));
        len += p_n_k(items_len, it);
    }

    printf("Will generate %d combinations\n", len);

    combination_pos = 0;
    for(it=1; it<=items_len; it++) {
        combinations_in_iteration = p_n_k(items_len, it);
        generator_init(generator, it);

        for(j=0; j<combinations_in_iteration; j++) {
            combo = (combination *)malloc(sizeof(struct combination));
            combo->len = it;
            combo->items=(item **)malloc(sizeof(item *)*it);
            for(pos=0; pos<it; pos++) {
                combo->items[pos] = items[generator[pos]];
            }

            work_combination(combo, combination_pos, len);

            free(combo->items);
            free(combo);

            combination_pos++;
            generator_next(generator, 0,items_len, it);
        }

        combinations_in_iteration *= items_len;
    }

    free(generator);
}

void free_combinations(combination ***pthis,unsigned long len) {
    unsigned long combination_pos;
    combination **combinations=*pthis;

    for(combination_pos=0; combination_pos<len; combination_pos++) {
        free(combinations[combination_pos]->items);
        free(combinations[combination_pos]);
    }

    free(combinations);
}

int item_cmp(item **pa,item **pb) {
    item *a=*pa;
    item *b=*pb;

    int ret = strncmp(a->k, b->k,
                      a->kl<b->kl ? a->kl : b->kl);
    if(ret == 0) {
        ret=a->kl-b->kl;
    }

    return ret;
}

void *item_fake_free(void *this) {}

void work_combination(combination *this,unsigned long combination_pos,unsigned long combination_len) {
    unsigned long pos, pos2;
    item *item1;
    item *item2;
    void *nvalue;
    static time_t time_start=0;
    time_t time_elapsed;
    time_t estimated_time_left=0;
    struct tm *atime;
    char atimef[30];

    combination *unduplicated;
    combination *sorted;

    unduplicate_combination(this, &unduplicated);
    sort_combination(unduplicated, &sorted);

    ptrie *trie;
    ptrie_new(item_fake_free, &trie);

    if(combination_pos == 0) {
        printf("RESET TIME\n");
        time_start=time(0);
    }

    if(combination_pos%100000 == 0) {
        time_elapsed = time(0) - time_start;
        estimated_time_left = (combination_len+1)/(combination_pos+1.0);
        estimated_time_left *= time_elapsed;

        char *dmp=0;
        concat(&dmp,"Iteration %ld (%lf)\n",combination_pos, (combination_len+1.0)/(combination_pos+1));
        combination_concat(&dmp, unduplicated);

        atime=localtime(&time_elapsed);
        strftime(atimef, sizeof(atimef), "%Y:%m:%d %H:%M:%S", atime);
        concat(&dmp, "Time elapsed: %s [%ld]\n", atimef,time_elapsed);

        atime=localtime(&estimated_time_left);
        strftime(atimef, sizeof(atimef), "%Y:%m:%d %H:%M:%S", atime);
        concat(&dmp, "Time left: %s [%ld]\n", atimef,estimated_time_left);
        printf("%s\n",dmp);
        free(dmp);

    }

    for(pos=0; pos<unduplicated->len; pos++) {
        item1 = unduplicated->items[pos];
        if(PR_OK != ptrie_set(trie, item1->kl, item1->k, item1->k))
        {
            error_1_cannot_set(this, item1);
        }
    }

    for(pos=0; pos<unduplicated->len; pos++) {
        item1 = unduplicated->items[pos];
        if(PR_OK != ptrie_get(trie, item1->kl, item1->k, &nvalue)) {
            error_2_cannot_get(this, item1);
        }
    }

    for(pos=0; pos<unduplicated->len; pos++) {
        item1 = unduplicated->items[pos];

        if(PR_OK !=ptrie_delete(trie, item1->kl, item1->k)) {
            error_3_1_cannot_delete(this, item1);
        }

        if(PR_ERROR != ptrie_get(trie, item1->kl, item1->k, &nvalue)) {
            error_3_2_did_get(this, item1);
        }

        for(pos2=0; pos2<unduplicated->len; pos2++) {
            item2 = unduplicated->items[pos2];
            if(item2 == item1) {
                continue;
            }

            if(PR_OK != ptrie_get(trie, item2->kl, item2->k, &nvalue)) {
                error_3_3_cannot_get(this, item1, item2);
            }
        }

        if(PR_OK != ptrie_set(trie, item1->kl, item1->k, item1->k)) {
            error_3_4_cannot_set(this, item1);
        }
    }

    free(sorted->items);
    free(sorted);

    free(unduplicated->items);
    free(unduplicated);

    ptrie_free(&trie);
}

void unduplicate_combination(combination *in,combination **pout) {
    short *will_be=(short *)malloc(sizeof(short)*in->len);
    unsigned long it,it2;
    unsigned int plen;
    combination *out;

    for(it=0; it<in->len; it++) {
        will_be[it]=1;
    }
    for(it=0; it<in->len-1; it++) {
        for(it2=it+1; it2<in->len; it2++) {
            if(in->items[it] == in->items[it2]) {
                will_be[it2]=0;
            }
        }
    }

    plen=0;
    for(it=0; it<in->len; it++) {
        if(will_be[it] == 1) {
            plen++;
        }
    }

    out=(combination *)malloc(sizeof(struct combination));
    out->len=plen;
    out->items=(item **)malloc(sizeof(struct item)*plen);

    for(it=0,it2=0; it<in->len; it++) {
        if(will_be[it]==1) {
            out->items[it2] = in->items[it];
            it2++;
        }
    }

    *pout=out;
    free(will_be);
}

void sort_combination(combination *in,combination **pout) {
    unsigned long it;
    combination *out;

    out=(combination *)malloc(sizeof(struct combination));
    out->len=in->len;
    out->items=(item **)malloc(sizeof(struct item)*(in->len));

    for(it=0; it<in->len; it++) {
        out->items[it] = in->items[it];
    }

    qsort(out->items,in->len,sizeof(item *), item_cmp);
    *pout=out;
}

void test_performance() {
    item **items;
    unsigned long items_len;

    generate_items(&items, &items_len);
    work_combinations(items, items_len);
    free_items(&items, items_len);
}

void error_1_cannot_set(combination *c,item *i) {
    char *str=0;
    concat(&str, "[ERROR] 1. Cannot set item into trie:\n# item: ");
    item_concat(&str, i);
    concat(&str, "# combination:\n");
    combination_concat(&str, c);

    printf("%s\n",str);
    CU_assertImplementation(0, 1, str, "test.c", "work_combination", 0);
}

void error_2_cannot_get(combination *c,item *i) {
    char *str=0;
    concat(&str, "[ERROR] 2. Cannot get item from trie:\n# item: ");
    item_concat(&str, i);
    concat(&str, "# combination:\n");
    combination_concat(&str, c);

    printf("%s\n",str);
    CU_assertImplementation(0, 1, str, "test.c", "work_combination", 0);
}

void error_3_1_cannot_delete(combination *c,item *i) {
    char *str=0;
    concat(&str, "[ERROR] 3.1. Cannot delete item from trie:\n# item: ");
    item_concat(&str, i);
    concat(&str, "# combination:\n");
    combination_concat(&str, c);

    printf("%s\n",str);
    CU_assertImplementation(0, 1, str, "test.c", "work_combination", 0);
}
void error_3_2_did_get(combination *c,item *i) {
    char *str=0;
    concat(&str, "[ERROR] 3.2. Did get item from trie:\n# item: ");
    item_concat(&str, i);
    concat(&str, "# combination:\n");
    combination_concat(&str, c);

    printf("%s\n",str);
    CU_assertImplementation(0, 1, str, "test.c", "work_combination", 0);
}
void error_3_3_cannot_get(combination *c,item *i,item *j) {
    char *str=0;
    concat(&str, "[ERROR] 3.2. Cannot get item from trie after removing other:\n# item deleted: ");
    item_concat(&str, i);
    concat(&str, "# item to get:");
    item_concat(&str,j);
    concat(&str, "# combination:\n");
    combination_concat(&str, c);

    printf("%s\n",str);
    CU_assertImplementation(0, 1, str, "test.c", "work_combination", 0);
}
void error_3_4_cannot_set(combination *c,item *i) {
    char *str=0;
    concat(&str, "[ERROR] 3.4. Cannot set back item into trie:\n# item: ");
    item_concat(&str, i);
    concat(&str, "# combination:\n");
    combination_concat(&str, c);

    printf("%s\n",str);
    CU_assertImplementation(0, 1, str, "test.c", "work_combination", 0);
}

void concat(char **pthis, char *format, ...) {
    char *pre=*pthis;
    int prelen = pre!=0 ? strlen(pre) : 0;
    char *now = 0;
    int ret;
    int length;
    va_list args;

    va_start(args, format);
    ret=vsnprintf(0, 0, format, args);
    va_end(args);

    length=ret + 1;
    now=(char *)malloc(sizeof(char)*(length+prelen+1));

    if(pre!=0) {
        strncpy(now, pre, prelen);
    }

    va_start(args, format);
    vsnprintf(now+prelen, length, format, args);
    va_end(args);
    now[prelen+ret]=0;

    if(pre!=0) {
        free(pre);
    }

    *pthis=now;
}

void combination_concat(char **pthis,combination *c) {

    unsigned long pos;
    concat(pthis,"len=%d\n",c->len);

    for(pos=0; pos<c->len; pos++) {
        concat(pthis, "### ");
        item_concat(pthis,c->items[pos]);
    }
}

void item_concat(char **pthis,item *i) {
    char *istr=(char *)malloc(sizeof(char)*(i->kl));
    strncpy(istr, i->k, i->kl);
    istr[i->kl]=0;

    concat(pthis, "%d <%s>\n",i->kl, istr);
    free(istr);

}

